﻿/////////////////////////////////////////////////////////////////////////////
// Name:        hmp4000_Simple_Example
//
// Purpose:     hmp4000 sample program to demonstrate using of HMP4000 series arbitrary generator and arb signal memory banks
//              by utilizing the R&S IVI.NET Hmp4000 driver in C#
//
// Author:      Miloslav Macko
// Created:     13-November-2018
// Copyright:   (c) Rohde & Schwarz, Munich
/////////////////////////////////////////////////////////////////////////////

using System;
using System.Threading;
using RohdeSchwarz.Hmp4000;

//If missing, add the reference from:
// 64-bit target application:
//"C:\Program Files\IVI Foundation\IVI\Microsoft.NET\Framework64\v4.0.30319\RohdeSchwarz.Hmp4000 <version>\RohdeSchwarz.Hmp4000.Fx40.dll"
// 32-bit target application:
//"C:\Program Files (x86)\IVI Foundation\IVI\Microsoft.NET\Framework32\v4.0.30319\RohdeSchwarz.Hmp4000 <version>\RohdeSchwarz.Hmp4000.Fx40.dll"

//--------------------------------------------------------------------------
// Prerequisites
//
//   This sample program needs IVI Shared Components and IVI.NET Shared Components being installed.
//   If not yet done, visit
//     http://www.ivifoundation.org/shared_components/Default.aspx
//
//   Download and install
//     - IVI Shared Components 2.6.0
//     - IVI.NET Shared Components 1.4.0
//     - Rohde & Schwarz Hmp4000 IVI.NET driver 2.6.2.0
//
//   32-bit: IviSharedComponents_2.6.0.exe
//           IviNetSharedComponents32_Fx20_1.4.0.exe
//           Hmp4000-ivi.net-x86-2_6_2_0.msi
//
//   64-bit: IviSharedComponents64_2.4.2.exe
//           IviNetSharedComponents64_Fx20_1.4.0.exe
//           Hmp4000-ivi.net-x64-2_6_2_0.msi
//
//--------------------------------------------------------------------------

namespace Hmp4000_ArbExample
{
    class Program
    {
        static void Main()
        {
            // Before running, change the resource name to fit your device:
            // For LAN interface use TCPIP::<IP_Address>::5025::SOCKET e.g. "TCPIP::10.64.1.36::5025::SOCKET"
            // For Virtual Com-port use the ASRL<PORT>::INSTR e.g. "ASRL10::INSTR"
            // For GPIB interface use the GPIB::<Address>::INSTR e.g. "GPIB::30::INSTR"
            // For USB-TMC interface use the USB::0x0AAD::<PID>::<SERIAL_NUMBER>::INSTR e.g. "USB0::0x0AAD::0x0117::100405::INSTR"
            Hmp4000 driver;
            Console.Out.Write("Driver session initializing ... ");
            try
            {
                //driver = new Hmp4000("TCPIP::10.64.0.144::5025::SOCKET", true, true, "Simulate=False");
                //driver = new Hmp4000("ASRL18::INSTR", true, true, "Simulate=False");
                //driver = new Hmp4000("GPIB::30::INSTR", true, true);
                driver = new Hmp4000("USB0::0x0AAD::0x0117::100405::INSTR", false, true);
                Console.WriteLine("finished");
                Console.Write("Instrument ID: ");
                Console.WriteLine(driver.System.IDQueryResponse);
                var selftestResult = driver.System.SelfTest();
                Console.WriteLine("Instrument Selftest: {0}", selftestResult.Message);
            }
            //in case of the initializing error the driver throws an Ivi.Driver.IOException
            catch (Ivi.Driver.IOException e)
            {
                Console.Out.WriteLine("\nERROR: Cannot initialize the driver session.\nDetails:");
                Console.Out.WriteLine(e.Message);
                Console.Out.Write("\n\nPress any key to finish ... ");
                Console.ReadKey();
                return;
            }
            try
            {
                // If speed is critical, switch the QueryInstrumentStatus status OFF
                // By default, the QueryInstrumentStatus value is ON
                driver.DriverOperation.QueryInstrumentStatus = true;

                // Select the channel 1. All further channel-commands are adressed to this channel
                Console.Write("Configuring Channel 1 ... ");
                driver.BasicOperation.SelectedChannel = Output.Channel1;
                driver.VoltageAndCurrent.OutputVoltageLevel = 1.23; // Voltage when the ARB is OFF
                driver.VoltageAndCurrent.OutputCurrentLevel = 0.123;
                driver.VoltageAndCurrent.OverVoltageProtectionLevel = 11.0;
                driver.BasicOperation.OutputEnabled = true;
                Console.WriteLine("finished");

                // Configure the arb signal
                double[] voltages = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 };
                double[] currents = { 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1 };
                double[] times = { 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5 };
                driver.ArbitraryGenerator.ConfigureArbitraryGeneratorWaveformData(voltages, currents, times);
                driver.ArbitraryGenerator.Repetition = 0; // infinite repetition
                driver.ArbitraryGenerator.DataTransferChannel = Channel.Channel1;
                driver.ArbitraryGenerator.StartChannel = Channel.Channel1;

                Console.WriteLine("Arb data 1 measurement loop:");
                driver.BasicOperation.OutputEnabled = true;
                for (int i = 0; i < 20; i++)
                {
                    var voltage = driver.Measurement.DCVoltage;
                    var current = driver.Measurement.DCCurrent;
                    Console.WriteLine("Voltage: {0:F3}, Current {1:F3}", voltage, current);
                    Thread.Sleep(50);
                }

                Console.Write("Stopping arb signal generation ...");
                driver.ArbitraryGenerator.StopChannel = Channel.Channel1;
                Console.WriteLine("finished");

                // Operation with arb signals and memory banks
                Console.Write("Saving the arb data 1 to memory bank 1 ...");
                driver.ArbitraryGenerator.Save = MemoryIndex.Index1;
                Console.WriteLine("finished");

                // Configure another arb signal and storing it to the memory bank 2
                voltages = new double[] { 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0 };
                currents = new double[] { 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1 };
                times = new double[] { 0.2, 0.2, 0.2, 0.2, 0.2, 0.2, 0.2, 0.2, 0.2, 0.2 };
                Console.Write("Saving the arb data 2 to memory bank 2 ...");
                driver.ArbitraryGenerator.Save = MemoryIndex.Index2;
                Console.WriteLine("finished");

                Console.Write("Recalling the Arb data 2 from the memory bank 2 ... ");
                driver.ArbitraryGenerator.Recall = MemoryIndex.Index2;
                Console.WriteLine("finished");

                driver.ArbitraryGenerator.Repetition = 0; // infinite repetition
                driver.ArbitraryGenerator.DataTransferChannel = Channel.Channel1;
                driver.ArbitraryGenerator.StartChannel = Channel.Channel1;
                Console.WriteLine("Arb data 2 measurement loop:");
                driver.BasicOperation.OutputEnabled = true;
                for (int i = 0; i < 20; i++)
                {
                    var voltage = driver.Measurement.DCVoltage;
                    var current = driver.Measurement.DCCurrent;
                    Console.WriteLine("Voltage: {0:F3}, Current {1:F3}", voltage, current);
                    Thread.Sleep(10);
                }

                Console.Write("Stopping arb signal generation ...");
                driver.ArbitraryGenerator.StopChannel = Channel.Channel1;
                Console.WriteLine("finished");

                Console.Write("Recalling the Arb data 1 from the memory bank 1 ... ");
                driver.ArbitraryGenerator.Recall = MemoryIndex.Index1;
                Console.WriteLine("finished");

                driver.ArbitraryGenerator.Repetition = 0; // infinite repetition
                driver.ArbitraryGenerator.DataTransferChannel = Channel.Channel1;
                driver.ArbitraryGenerator.StartChannel = Channel.Channel1;
                Console.WriteLine("Arb data 1 measurement loop:");
                driver.BasicOperation.OutputEnabled = true;
                for (int i = 0; i < 20; i++)
                {
                    var voltage = driver.Measurement.DCVoltage;
                    var current = driver.Measurement.DCCurrent;
                    Console.WriteLine("Voltage: {0:F3}, Current {1:F3}", voltage, current);
                    Thread.Sleep(50);
                }

                Console.Write("Stopping arb signal generation ...");
                driver.ArbitraryGenerator.StopChannel = Channel.Channel1;
                Console.WriteLine("finished");

                driver.System.SystemLocal();
            }
            //driver IO error
            catch (Ivi.Driver.IOException e)
            {
                Console.WriteLine("Driver error occured:");
                Console.WriteLine(e.Message);
            }

            //driver function is not supported by this sensor
            catch (Ivi.Driver.OperationNotSupportedException e)
            {
                Console.WriteLine("Instrument doesn't support the function:");
                Console.WriteLine(e.Message);
            }

            //when a task takes longer than the defined timeout
            catch (Ivi.Driver.MaxTimeExceededException e)
            {
                Console.WriteLine("Operation took longer than the maximum defined time:");
                Console.WriteLine(e.Message);
            }

            //if the instrument returns an error in the error queue
            catch (Ivi.Driver.InstrumentStatusException e)
            {
                Console.WriteLine("Instrument system error occured:");
                Console.WriteLine(e.Message);
            }

            //if the instrument returns an error in the error queue
            catch (Ivi.Driver.SelectorNameException e)
            {
                Console.WriteLine("Invalid selector name used:");
                Console.WriteLine(e.Message);
            }

            finally
            {
                Console.Out.Write("\n\nPress any key to finish ... ");
                Console.ReadKey();
            }
        }
    }
}